@@ -1,12 +1,12 @@
 from aqueue.display import SetDescFn
 from aqueue.processing import async_run_queue, run_queue
-from aqueue.queue import EnqueueFn, Item, QueueTypeName
+from aqueue.queue import EnqueueFn, Item, Ordering
 
 __all__ = (
     "SetDescFn",
     "EnqueueFn",
     "Item",
-    "QueueTypeName",
+    "Ordering",
     "run_queue",
     "async_run_queue",
 )
